package org.openxma.dsl.generator.helper;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.Style;
import org.openxma.dsl.generator.GeneratorProperties;
import org.openxma.dsl.pom.model.AttachmentOwnPosition;
import org.openxma.dsl.pom.model.AttachmentProperty;
import org.openxma.dsl.pom.model.AttachmentSpecification;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.FieldPart;
import org.openxma.dsl.pom.model.FieldPartSpecification;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.HeightProperty;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.WidthProperty;
import org.openxma.dsl.pom.util.LayoutData;

import at.spardat.xma.guidesign.XMAWidget;

public class LayoutDataExtension {
	
	public static void setDefaultWidth(GuiElement guiElement, Style style) {
		if (guiElement instanceof IElementWithLayoutData) {
			IElementWithLayoutData elementWithLayoutData = (IElementWithLayoutData)guiElement;
			LayoutData layoutData = new LayoutData(elementWithLayoutData);
			if (layoutData.getWidth() == 0 || isWidthFromLayoutStrategy(guiElement)) {														
				int width = GeneratorProperties.getLayoutStrategy().getDefaultGuiElementWidth(guiElement,style);
				if (width != -1) {
					layoutData.setWidth(width);	
					setWidthIsFromLayoutStrategy(guiElement);					
				}
			}			
		}		
	}
	
	public static boolean isWidthFromLayoutStrategy(EObject obj) {
		Object value = EObjectPropertiesAdapter.getProperty(obj,"widthIsFromLayoutStrategy");
		if (value instanceof Boolean) {
			return ((Boolean)value).booleanValue();
		}
		return false;
	}
	
	public static void setWidthIsFromLayoutStrategy(EObject obj) {
		if (obj != null) {
			EObjectPropertiesAdapter.setProperty(obj,"widthIsFromLayoutStrategy",new Boolean(true));
		}
	}
	
	/**
	 * Get the layoutdata for a GuiElement
	 */	
	public static LayoutData getLayoutData(GuiElement guiElement) {
		
		if (PresentationModelExtension.canModelElementHaveStyle(guiElement)) {
			return new LayoutData((IElementWithLayoutData)guiElement);
		}
		return null;
	}
	
	/**
	 * Get the layoutdata for a XMAWidget
	 */		
	public static LayoutData getLayoutData(XMAWidget xmaWidget) {
		GuiElement guiElement = WidgetExtension.getGuiElementProperty(xmaWidget);
		if (guiElement == null) return null;
		return getLayoutData(guiElement);
	}
		
    /**
	 *
     * @return the top attachment
     */
	public static AttachmentSpecification getTopAttachment(LayoutData layoutData) {
		if (layoutData != null) {
			return layoutData.getTopAttachment();
		} return null;
	}
	
    /**
	 *
     * @return the left attachment
     */
	public static AttachmentSpecification getLeftAttachment(LayoutData layoutData) {
		if (layoutData != null) {		
			return layoutData.getLeftAttachment();
		} return null;
	}
	
    /**
     *
     * @return the right attachment
     */
	public static AttachmentSpecification getRightAttachment(LayoutData layoutData) {
		if (layoutData != null) {		
			return layoutData.getRightAttachment();
		}
		return null;		
	}
	
    /**
     *
     * @return the bottom attachment
     */
	public static AttachmentSpecification getBottomAttachment(LayoutData layoutData) {
		if (layoutData != null) {			
			return layoutData.getBottomAttachment();
		} return null;
	}
	
	
	public static AttachmentSpecification getAttachment(FieldPartSpecification fieldPartSpecification, final AttachmentOwnPosition ownPosition) {
		List<AttachmentProperty> props = LayoutData.getAttachments(fieldPartSpecification.getLayoutProperties());
		return LayoutData.getAttachment(props, ownPosition);
	}	
	
	
    /**
	 *
    * @return the top attachment
    */
	public static AttachmentSpecification getTopAttachment(GuiElement guiElement, FieldPart fieldPart, FieldPartSpecification fieldPartSpecification) {
		if (guiElement == null) return null;
		if (guiElement instanceof CustomizeableField) {
			if (fieldPartSpecification != null) {		
				AttachmentSpecification leftFromPart = getAttachment(fieldPartSpecification,AttachmentOwnPosition.TOP);
				if (leftFromPart != null) return leftFromPart;
			}
			if (fieldPart!=null && fieldPart.getValue() != FieldPart.CONTROL.getValue()) return null;
		}			
		LayoutData layoutData = getLayoutData(guiElement);			
		if (layoutData != null) {
			return layoutData.getTopAttachment();
		} return null;
	}
	
   /**
	 *
    * @return the left attachment
    */
	public static AttachmentSpecification getLeftAttachment(GuiElement guiElement, FieldPartSpecification fieldPartSpecification, Boolean useParentAttachment) {
		if (guiElement == null) return null;
		if (guiElement instanceof CustomizeableField) {
			if (fieldPartSpecification != null) {		
				AttachmentSpecification leftFromPart = getAttachment(fieldPartSpecification,AttachmentOwnPosition.LEFT);
				if (leftFromPart != null) return leftFromPart;							
			}										
		}
		if (useParentAttachment != null && useParentAttachment == false) return null;				
		LayoutData layoutData = getLayoutData(guiElement);			
		if (layoutData != null) {		
			return layoutData.getLeftAttachment();
		} return null;
	}
	
   /**
    *
    * @return the right attachment
    */
	public static AttachmentSpecification getRightAttachment(GuiElement guiElement, FieldPartSpecification fieldPartSpecification, Boolean useParentAttachment) {
		if (guiElement == null) return null;		
		if (guiElement instanceof CustomizeableField) {
			if (fieldPartSpecification != null) {		
				AttachmentSpecification leftFromPart = getAttachment(fieldPartSpecification,AttachmentOwnPosition.RIGHT);
				if (leftFromPart != null) return leftFromPart;							
			}					
		}	
		if (useParentAttachment != null && useParentAttachment == false) return null;		
		LayoutData layoutData = getLayoutData(guiElement);			
		if (layoutData != null) {		
			return layoutData.getRightAttachment();
		}
		return null;		
	}
	
   /**
    *
    * @return the bottom attachment
    */
	public static AttachmentSpecification getBottomAttachment(GuiElement guiElement, FieldPart fieldPart, FieldPartSpecification fieldPartSpecification) {
		if (guiElement == null) return null;
		if (guiElement instanceof CustomizeableField) {		
			if (fieldPartSpecification != null) {		
				AttachmentSpecification leftFromPart = getAttachment(fieldPartSpecification,AttachmentOwnPosition.BOTTOM);
				if (leftFromPart != null) return leftFromPart;
			}
			if (fieldPart!=null && fieldPart.getValue() != FieldPart.CONTROL.getValue()) return null;
		}
		LayoutData layoutData = getLayoutData(guiElement);		
		if (layoutData != null) {			
			return layoutData.getBottomAttachment();
		} return null;
	}
	
	public static int getWidth(LayoutData layoutData) {
		if (layoutData == null) return 0;		
		for (Object obj : layoutData.getProperties()) {
			if (obj instanceof WidthProperty) {			    
				return ((WidthProperty)obj).getWidth();
			}
		}
		return 0;
	}
	
	public static int getHeight(LayoutData layoutData) {
		if (layoutData == null) return 0;		
		for (Object obj : layoutData.getProperties()) {
			if (obj instanceof HeightProperty) {
				return ((HeightProperty)obj).getHeight();
			}
		}
		return 0;
	}	
	
	public static int getWidth(GuiElement guiElement) {
		if (guiElement == null) return 0;
		LayoutData layoutData = getLayoutData(guiElement);
		return getWidth(layoutData);
	}
	
	public static int getHeight(GuiElement guiElement) {
		if (guiElement == null) return 0;
		LayoutData layoutData = getLayoutData(guiElement);	
		return getHeight(layoutData);
	}	
}
