package org.openxma.dsl.generator.helper;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.generator.GeneratorProperties;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.TabulatorPosition;
import org.openxma.dsl.pom.scoping.PomElementCollector;


public class TabulatorExtension {		
	
	public static void initTabulator(Composite composite, String style) {
		
		ComposeData composeData = FormDataExtension.getComposeData(composite);
		
		if (FormDataExtension.hasComposeData(composite)==false) return;
		
		// Use explicitly defined tabulators if available
		if (composeData != null && composeData.getTabulators().size()>0 && isTabulatorFromLayoutStrategy(composite)==false ) {		
			return;
		}
		
		Composite parentComposite = PomElementCollector.getContainerWithComposeData(composite);
		if (parentComposite != null) {
			ComposeData pcd = FormDataExtension.getComposeData(parentComposite);
			if (pcd != null && pcd.getTabulators().size() > 0 && isTabulatorFromLayoutStrategy(parentComposite) == false) {
				if (composeData == null) {
					composeData = PomFactory.eINSTANCE.createComposeData();
					FormDataExtension.setComposeData(composite,composeData);
				}
				composeData.getTabulators().addAll(EcoreUtil.copyAll(pcd.getTabulators()));
				return;
			}
		}
		
		List<TabulatorPosition> definedTabulatorPositions = null;
		if (composeData != null) {
			definedTabulatorPositions = composeData.getTabulators();
		}
		
		List<TabulatorPosition> tabs = GeneratorProperties.getLayoutStrategy().getTabulatorPositions(composite, definedTabulatorPositions, style);	
		if (tabs != null && tabs.size()>0) {			
			if (composeData == null) {
				composeData = PomFactory.eINSTANCE.createComposeData();	
				FormDataExtension.setComposeData(composite,composeData);
			} 		
			setTabulatorIsFromLayoutStrategy(composite);
			composeData.getTabulators().addAll(tabs);
		}
	}
	
	private static boolean isTabulatorFromLayoutStrategy(EObject obj) {
		Object value = EObjectPropertiesAdapter.getProperty(obj,"tabulatorIsFromLayoutStrategy");
		if (value instanceof Boolean) {
			return ((Boolean)value).booleanValue();
		}
		return false;
	}
	
	private static void setTabulatorIsFromLayoutStrategy(EObject obj) {
		if (obj != null) {
			EObjectPropertiesAdapter.setProperty(obj,"tabulatorIsFromLayoutStrategy",new Boolean(true));
		}
	}
}
