package org.openxma.dsl.generator.helper.style;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.generator.helper.PresentationModelExtension;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.util.StyleAccess;

public class ApplyCombinedStyles {
    
    /**
     * Apply the previously combined styles to the model element. In general this is done in cast
     * a property is not already set on the model element.
     * 
     * Sometimes styles do something else (usually more) then setting properties of presentation model
     * elements. These styles usually can not be applied here. To apply them e.g. a access to the 
     * platform specific model (.xma) may be required.
     * 
     * @param obj The presentation model element for which the style will be applied.
     * @param styleProperties All available style properties contained in a map.
     */
    public static void applyToPresentationModelObject(EObject obj, Map<Integer, StyleSpecificationProperty> styleProperties) {
        
        /* Page */
        if (obj instanceof Page) {
            Page page = (Page)obj;                  
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);      
            if (stylePropertyOther != null && page.getComposeLayout() == null) {             
                page.setComposeLayout(createComposeDataOnDemandAndApplyStyleProperties(styleProperties));
            }            
        }        
        
        /* Content */
        if (obj instanceof Content) {
            Content content = (Content)obj;                  
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);      
            if (stylePropertyOther != null && content.getComposeLayout() == null) {             
                content.setComposeLayout(createComposeDataOnDemandAndApplyStyleProperties(styleProperties));
            }            
        }
        
        /* ComposeData */
        if (obj instanceof ComposeData) {            
            ComposeData composeData = (ComposeData)obj;            
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);
            if (stylePropertyOther != null) { 
                if (composeData.getMargin() == null && stylePropertyOther.getMargin() != null) {
                    composeData.setMargin(stylePropertyOther.getMargin());                
                }
                if (composeData.getSpacing() == null && stylePropertyOther.getSpacing() != null) {
                    composeData.setSpacing(stylePropertyOther.getSpacing());
                }      
            }
        }        
    }
    
    /**
     * Create ComposeData in case that there are styles to set and apply the styles.
     * 
     * @param styleProperties
     * @return
     */
    private static ComposeData createComposeDataOnDemandAndApplyStyleProperties(Map<Integer, StyleSpecificationProperty> styleProperties) {
        StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);        
        if (stylePropertyOther != null && (stylePropertyOther.getMargin() != null || stylePropertyOther.getSpacing() != null)) {            
            // Create the compositeLayout (because there are properties to set from styles)                                        
            ComposeData composeData = PomFactory.eINSTANCE.createComposeData();
            StyleAccess.setCombinedStyleProperty(composeData, styleProperties);                                                
            applyToPresentationModelObject(composeData,styleProperties);
            return composeData;
        }        
        return null;        
    }

}
