package org.openxma.dsl.generator.component;

import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.emf.mwe.core.WorkflowContext;
import org.eclipse.emf.mwe.core.issues.Issues;
import org.eclipse.emf.mwe.core.lib.AbstractWorkflowComponent2;
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor;

// decompiled with JAD from oAW-4.3.1
public class DirectoryCleaner extends AbstractWorkflowComponent2 {

    private static final String COMPONENT_NAME = "Directory Cleaner";
    private static final Log LOG = LogFactory.getLog(org.openxma.dsl.generator.component.DirectoryCleaner.class);
    private String directory;
    private final Set excludes;
    private final String defaultExcludes[] = { ".CVS", ".svn" };
    private boolean useDefaultExcludes;

    public DirectoryCleaner() {
        excludes = new HashSet();
        useDefaultExcludes = true;
    }

    /**
     * @deprecated Method setDirectories is deprecated
     */
    @Deprecated
    public void setDirectories(String directory) {
        LOG.warn("Property 'directories' is deprecated. Use 'directory' instead.");
        setDirectory(directory);
    }

    public void setDirectory(String directory) {
        this.directory = directory;
    }

    @Override
    public String getLogMessage() {
        return new StringBuilder().append("cleaning directory '").append(directory).append("'").toString();
    }

    @Override
    protected void invokeInternal(WorkflowContext model, ProgressMonitor monitor, Issues issues) {
        if (directory != null) {
            StringTokenizer st = new StringTokenizer(directory, ",");
            do {
                if (!st.hasMoreElements()) {
                    break;
                }
                String dir = st.nextToken().trim();
                File f = new File(dir);
                if (f.exists() && f.isDirectory()) {
                    LOG.info(new StringBuilder().append("Cleaning ").append(f.getAbsolutePath()).toString());
                    File contents[] = f.listFiles();
                    int j = 0;
                    while (j < contents.length) {
                        File file = contents[j];
                        if (!delete(file)) {
                            LOG.error(new StringBuilder().append("Couldn't delete ").append(file.getAbsolutePath()).toString());
                        }
                        j++;
                    }
                }
            } while (true);
        }
    }

    @Override
    protected void checkConfigurationInternal(Issues issues) {
        if (directory == null) {
            issues.addWarning("No directories specified!");
        }
    }

    public boolean delete(File file) {
        boolean totalSuccess = true;
        if (file.isDirectory()) {
            String children[] = file.list();
            for (int i = 0; i < children.length; i++) {
                if (isExcluded(file)) {
                    continue;
                }
                boolean success = delete(new File(file, children[i]));
                totalSuccess = totalSuccess && success;
                // if (!success) {
                //    return false;
                // }
            }

        }

        if (!isExcluded(file)) {
            return file.delete() && totalSuccess;
        } else {
            return totalSuccess;
        }
    }

    public boolean isUseDefaultExcludes() {
        return useDefaultExcludes;
    }

    public void setUseDefaultExcludes(boolean useDefaultExcludes) {
        this.useDefaultExcludes = useDefaultExcludes;
    }

    public void addExclude(String exclude) {
        excludes.add(exclude);
    }

    @Override
    public String getComponentName() {
        return "Directory Cleaner";
    }

    private boolean isExcluded(File file) {
        if (file == null) {
            throw new IllegalArgumentException();
        }
        String name = file.getName();
        if (useDefaultExcludes) {
            String arr[] = defaultExcludes;
            int len = arr.length;
            for (int i = 0; i < len; i++) {
                String excl = arr[i];
                if (name.equals(excl)) {
                    return true;
                }
            }

        }
        return excludes.contains(name);
    }

}
